import argparse
import os
import html
import textwrap
from time import strftime, localtime
from replbuilder import ReplCommand
from .get_story import print_story


def get_comment_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('pointer', nargs="?", type=int, help="Get comments tree under previous listed pointer")
    parser.add_argument('-b', '--breadth', type=int, default=5, help="How many direct replies to get for each level")
    parser.add_argument('-d', '--depth', type=int, default=5, help="Max depth of comment chain traversed")
    parser.add_argument('-l', '--limit', type=int, default=100, help="Max number of comments retrieved")
    parser.add_argument('-i', '--item-id', type=int, help="Get comments tree under a specific item-id. must be provided if pointer isn't")
    return parser


def get_comments(args, context):
    if args.pointer:
        item_id = context.current_pointers[args.pointer]
    elif args.item_id:
        item_id = args.item_id
    else:
        raise ValueError("Must either provide --pointer or --item-id")
    context.clear_pointers()
    context.curr_pointer = 1
    context.store_item(item_id)
    item = context.loaded_items[item_id]
    if item["type"] == "comment":
        level = 0
        parent_id = item["parent"]
        context.store_item(parent_id)
        parent_item = context.loaded_items[parent_id]
        context.store_pointer(-1, parent_id)
        if parent_item["type"] == "comment":
            print("\033[1;32mPARENT COMMENT\033[0m")
            print_comment(-1, 0, parent_item)
        else:
            print("\033[1;33mPOINTER -1: \033[1;32mPARENT STORY\033[0m")
            print_story(parent_item)
    else:
        level = -1
        print("\033[1;33mPOINTER -1: \033[1;32mPARENT STORY\033[0m")
        print_story(item)
        context.store_pointer(-1, item_id)
    print("\033[1;32m{pointer: <19} | {text}\033[0m".format(pointer="POINTER/AUTHOR", text="COMMENTS"))
    get_comment_tree(context, item_id, args.breadth, args.depth, args.limit, level)


def print_comment(pointer, level, item):
    if "text" not in item:
        return
    comment = item["text"]
    comment = html.unescape(comment)
    comment = comment.replace("<i>", "\033[3m")
    comment = comment.replace("</i>", "\033[0m")
    comment_lines = comment.split("<p>")
    term_width = os.get_terminal_size().columns
    text_width = term_width - 30 - level * 5

    comment_lines_lines = [textwrap.wrap(l, text_width) for l in comment_lines]
    comment_lines = [l for subl in comment_lines_lines for l in subl]
    while len(comment_lines) < 3:
        comment_lines.append("")
    comment_time = strftime('%Y-%m-%d %H:%M:%S', localtime(item["time"]))
    pad = 19 + level * 6
    print("\033[1;33m{pointer: <{pad}}\033[0m | {text}".format(pointer=pointer, pad=pad, text=comment_lines[0]))
    print("\033[1;36m{author: <{pad}}\033[0m | {text}".format(author=item["by"], pad=pad, text=comment_lines[1]))
    print("{time: <{pad}} | {text}".format(time=comment_time, pad=pad, text=comment_lines[2]))
    for line in comment_lines[3:]:
        print(" "*(pad+1) + "| " + line)
    print()


def get_comment_tree(context, item_id, breadth=5, depth=5, limit=100, level=0):
    # DFS Search
    # breadth: search only $breadth top rated direct comments
    # depth: search only $depth level commands
    # limit: only retrieve $limit commands
    if level > depth or context.curr_pointer >= limit:
        return
    context.store_item(item_id)
    item = context.loaded_items[item_id]
    if item["type"] == "comment":
        print_comment(context.curr_pointer, level, item)
        context.store_pointer(context.curr_pointer, item_id)
        context.curr_pointer += 1
    if "kids" in item:
        for kid_id in item["kids"][:breadth]:
            get_comment_tree(context, kid_id, breadth, depth, limit, level+1)


get_comments_command = ReplCommand("get_comments", get_comment_parser(), get_comments, "Get comments by pointer, works with both stories and comments, uses DFS to construct a comment tree, control the recusion parameters with breadth, depth and limit arguments", use_context=True)



import argparse
import textwrap
import html
from time import strftime, localtime
from replbuilder import ReplCommand
from .get_comments import print_comment


def get_author_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('pointer', nargs="?", type=int, help="Get the comment belonging to previous listed pointer")
    parser.add_argument('-a', '--author-name', type=str, help="Get author by name, case sensitive, must be provided if pointer isn't")
    parser.add_argument('-s', '--show-submits', action="store_true", help="Show the most recent n submits belonging to this user")
    parser.add_argument('-n', '--number', type=int, default=5, help="Number of submits to show, default to 5")
    return parser


def print_about(text):
    text = html.unescape(text)
    text = text.replace("<i>", "")
    text = text.replace("</i>", "")
    lines = text.split("<p>")
    for line in lines:
        print("\033[3;33m{}\033[0m".format(line))
        print()


def print_story_with_pointer(pointer, story):
    story_lines = []
    story_lines.append("\033[1;36m{}\033[0m".format(story["title"]))
    story_lines.append(strftime('%Y-%m-%d %H:%M:%S', localtime(story["time"])))
    if "url" in story:
        story_lines.append("FULL URL: \033[4;35m{}\033[0m".format(story["url"]))
    print("\033[1;33m{pointer: <19}\033[0m | {text}".format(pointer=pointer, text=story_lines[0]))
    print("\033[1;36m{author: <19}\033[0m | {text}".format(author=story["by"], text=story_lines[1]))
    for line in story_lines[2:]:
        print(" "*20 + "| " + line)
    if "text" in story:
        text = story["text"]
        text = html.unescape(text)
        text = text.replace("<i>", "\033[3m")
        text = text.replace("</i>", "\033[0m")
        lines = text.split("<p>")
        text_width = os.get_terminal_size().columns - 30
        lines_lines = [textwrap.wrap(l, text_width) for l in lines]
        lines = [l for subl in lines_lines for l in subl]
        for line in lines:
            print(" "*20 + "| " + line)
    print()


def get_author(args, context):
    if not args.pointer and not args.author_name:
        raise ValueError("Must either provide --pointer or --item-id")
    if args.pointer:
        item_id = context.current_pointers[args.pointer]
        context.store_item(item_id)
        item = context.loaded_items[item_id]
        author_name = item["by"]
    else:
        author_name = args.author_name
    author_info = context.get_author_info(author_name)
    print("\033[1;36mUSER: {}\033[0m".format(author_name))
    if "about" in author_info:
        print_about(author_info["about"])
    print("TOTAL SUBMITS : {}".format(len(author_info["submitted"])))
    print("KARMA         : {}".format(author_info["karma"]))
    if args.show_submits:
        print("\033[1;32mDISPLAYING RECENT SUBMISSIONS\033[0m")
        print("\033[1;32m{pointer: <19} | {text}\033[0m".format(pointer="POINTER/AUTHOR", text="STORY/COMMENTS"))
        recent_items = author_info["submitted"][:args.number]
        context.clear_pointers()
        pointer = 0
        for item_id in recent_items:
            pointer += 1
            context.store_pointer(pointer, item_id)
            context.store_item(item_id)
            item = context.loaded_items[item_id]
            if ("dead" in item and item["dead"]) or ("deleted" in item and item["deleted"]):
                print()
                print("\033[1;31m{pointer: <19} | {text}\033[0m".format(pointer=pointer, text="DELETED"))
                print()
                continue
            if item["type"] == "comment":
                print_comment(pointer, 0, item)
            else:
                print_story_with_pointer(pointer, item)


get_author_command = ReplCommand("get_author", get_author_parser(), get_author, "Get the author by pointer or name, can also show recent submissions", use_context=True)



import argparse
import os
from replbuilder import ReplCommand
from urllib.parse import urlsplit
from time import strftime, localtime
from datetime import datetime


HN_LINER_SHORT_TOP = "\033[1;33m{rank: <7}| {title}\033[0m"
HN_LINER_SHORT_BOTTOM = "{rank: <7}| \033[0;36m{author: <24}\033[0m | {score: <5} | {comment: <7} | {age: <10} | {base_url}"
HN_LINER_SHORT_TITLE = "\033[1;32m{rank: <7}| {author: <24} | {score: <5} | {comment: <7} | {age: <10} | {base_url}\033[0m"
HN_STORY_LINER = "\033[1;33m{rank: <7}\033[0m| {title: <80} | {score: <5} | {comment: <7} | {age: <10} | {author: <20} | {base_url}"
HN_STORY_TITLE = "\033[1;32m{rank: <7}| {title: <80} | {score: <5} | {comment: <7} | {age: <10} | {author: <20} | {base_url}\033[0m"


def get_age_str(timestamp):
    diff = int(datetime.now().timestamp() - timestamp)
    if diff < 300:
        return "just now"
    if diff < 3600:
        return "{} minutes".format(diff // 60)
    if diff < 86400:
        return "{} hours".format(diff // 3600)
    return "{} days".format(diff // 86400)


def get_front_page_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--type', type=str, default="TOP", help="HN Story types, default to TOP, also try ASK, NEW, SHOW, JOB")
    parser.add_argument('-s', '--start-index', type=int, default=0, help="Start from index, default to 0")
    parser.add_argument('-n', '--number', type=int, default=30, help="Get n number of stories, default to 30")
    return parser


def print_story_line(rank, item, width):
    age = get_age_str(item["time"])
    base_url = "NONE"
    if "url" in item:
        base_url = urlsplit(item["url"]).netloc
    score = item["score"] if "score" in item else "N/A"
    comment = item["descendants"] if "descendants" in item else "N/A"
    author = item["by"]
    title = item["title"]
    if width >= 155:
        print(HN_STORY_LINER.format(rank=rank, title=item["title"], score=score, comment=comment, age=age, author=item["by"], base_url=base_url))
    else:
        print(HN_LINER_SHORT_TOP.format(rank=rank, title=title))
        print(HN_LINER_SHORT_BOTTOM.format(rank="", author=author, score=score, comment=comment, age=age, base_url=base_url))


def get_front_page(args, context):
    context.clear_pointers()
    context.get_story_list(args.type)
    stories_of_interest = context.story_list[args.start_index:args.start_index+args.number]

    width = os.get_terminal_size().columns
    if width >= 155:
        print(HN_STORY_TITLE.format(rank="POINTER", title="TITLE", score="SCORE", comment="COMMENT", age="AGE", author="AUTHOR", base_url="BASE URL"))
    else:
        print(HN_LINER_SHORT_TITLE.format(rank="POINTER", author="AUTHOR", score="SCORE", comment="COMMENT", age="AGE", base_url="BASE URL"))
    rank = 0
    for story_id in stories_of_interest:
        rank += 1
        context.store_item(story_id)
        print_story_line(rank, context.loaded_items[story_id], width)
        context.store_pointer(rank, story_id)


get_front_page_command = ReplCommand("get_front_page", get_front_page_parser(), get_front_page, "Get the front page of Hacker News", use_context=True)



import argparse
from .hnclient import HNClient
from replbuilder import ReplCommand


class HNContext:

    def __init__(self):
        self.client = HNClient()
        self.current_pointers = {}
        self.story_list = []
        self.loaded_items = {}
        self.call_count = 0

    def get_story_list(self, story_type="TOP"):
        self.call_count += 1
        self.story_list = self.client.get_stories(story_type=story_type)

    def store_item(self, item_id):
        if item_id in self.loaded_items:
            return
        self.call_count += 1
        item = self.client.get_item(item_id)
        self.loaded_items[item_id] = item

    def get_author_info(self, author_name):
        return self.client.get_author(author_name)

    def store_pointer(self, index, item_id):
        self.current_pointers[index] = item_id

    def clear_pointers(self):
        self.current_pointers = {}

    def get_cache(self, args):
        print("LOADED ITEMS COUNT : {}".format(len(self.loaded_items)))
        print("TOTAL HN API CALLS : {}".format(self.call_count))
        print("POINTERS COUNT     : {}".format(len(self.current_pointers)))

    def clear_cache(self, args):
        self.loaded_items = {}
        self.call_count = 0
        self.current_pointers = {}

    def get_context_commands(self):
        return [
            ReplCommand("get_cache", argparse.ArgumentParser(), self.get_cache, "See stored item count and call count"),
            ReplCommand("clear_cache", argparse.ArgumentParser(), self.clear_cache, "Remove all cache"),
        ]



import argparse
from replbuilder import ReplCommand
from time import strftime, localtime
import html


def print_text(text):
    text = html.unescape(text)
    text = text.replace("<i>", "\033[3m")
    text = text.replace("</i>", "\033[0m")
    lines = text.split("<p>")
    for line in lines:
        print(line)
        print()


def get_story_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('pointer', nargs="?", type=int, help="Get the story listed by rank")
    parser.add_argument('-i', '--story-id', type=int, help="Get the story listed by id, this must be provided if the pointer isn't")
    return parser


def print_story(story):
    print("\033[1;36m{}\033[0m".format(story["title"]))
    print(strftime('%Y-%m-%d %H:%M:%S', localtime(story["time"])))
    print("AUTHOR: {}".format(story["by"]))
    if "url" in story:
        print("FULL URL: \033[4;35m{}\033[0m".format(story["url"]))
    if "text" in story:
        print()
        print_text(story["text"])


def get_story(args, context):
    if args.pointer:
        story_id = context.current_pointers[args.pointer]
    elif args.story_id:
        story_id = args.story_id
    else:
        raise ValueError("Must either provide --pointer or --story-id")
    context.store_item(story_id)
    story = context.loaded_items[story_id]
    if story["type"] != "story":
        raise ValueError("Provided pointer or id is not a story")
    print_story(story)


get_story_command = ReplCommand("get_story", get_story_parser(), get_story, "Get story by pointer shown, use get_comments instead if you also want to see comments", use_context=True)



from replbuilder import ReplRunner
from hnterminal.hncontext import HNContext
from hnterminal.commands.get_front_page import get_front_page_command
from hnterminal.commands.get_story import get_story_command
from hnterminal.commands.get_comments import get_comments_command
from hnterminal.commands.get_author import get_author_command


def build_cli():
    hn_context = HNContext()
    cli_runner = ReplRunner("hnterminal", hn_context)
    cli_runner.add_commands([
        get_front_page_command,
        get_story_command,
        get_comments_command,
        get_author_command], namespace="Read")
    cli_runner.add_commands(hn_context.get_context_commands(), namespace="Cache")
    return cli_runner


def main():
    cli_runner = build_cli()
    cli_runner.run()


if __name__ == "__main__":
    main()



from urllib import request
import json
import html


class HNClient:

    HN_FIREBASE_URL = "https://hacker-news.firebaseio.com/v0/"
    STORY_TYPES = {
        "TOP": "topstories.json",
        "NEW": "newstories.json",
        "ASK": "askstories.json",
        "JOB": "jobstories.json",
        "SHOW": "showstories.json",
    }

    def get_stories(self, story_type="TOP"):
        if story_type not in HNClient.STORY_TYPES:
            raise ValueError("Story type must be one of {}".format(list(HNClient.STORY_TYPES.keys())))
        request_url = HNClient.HN_FIREBASE_URL + HNClient.STORY_TYPES[story_type]
        stories = json.loads(request.urlopen(request_url).read())
        return stories

    def get_item(self, item_id):
        request_url = HNClient.HN_FIREBASE_URL + "item/{}.json".format(item_id)
        item_info = json.loads(request.urlopen(request_url).read())
        return item_info

    def get_author(self, author_name):
        request_url = HNClient.HN_FIREBASE_URL + "user/{}.json".format(author_name)
        author_info = json.loads(request.urlopen(request_url).read())
        return author_info

